#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>          /* See NOTES */
#include <sys/socket.h>

#include "uartmultidropprotocol.hpp"


void build_led_message(UartMultidropProtocol::Frame & frame, const int _dest, const int _fader, const int _ledno, const int _mask, const int _value)
{
    frame.source  = 0x01;
    frame.dest    = _dest;
    frame.flags   = 0x45;
    frame.size    = 6;
    frame.data[0] = 'L';
    frame.data[1] = _fader;
    frame.data[2] = _ledno;
    frame.data[3] = _mask;
    frame.data[4] = (unsigned char)(_value>>8);
    frame.data[5] = (unsigned char)_value;
}

void build_observer_message(UartMultidropProtocol::Frame & frame)
{
    frame.source  = 0x01;
    frame.dest    = 0xff;
    frame.flags   = 0x45;
    frame.size    = 1;
    frame.data[0] = 'R';
}

int main (int argc, char **argv)
{
    UartMultidropProtocol::UmpInterface serialWingInterface;
    if (argc > 1)
    {
	serialWingInterface.setSerialName(argv[1]);
    }
    else
    {
	serialWingInterface.setSerialName("/dev/ttyUSB0");
    }

    int sockets[2];
    if (socketpair(AF_UNIX, SOCK_DGRAM, 0, sockets))
    {
	perror("socketpair");
	return 1;
    }
    printf ("%d, %d\n", sockets[0],  sockets[1]);
    serialWingInterface.setFrameSocket(sockets[0]);
    if (!serialWingInterface.start())
    {
	printf ("failed to start serial interface\n");
	return 1;
    }

    while (true)
    {
	fd_set read_fds;
	FD_ZERO(&read_fds);
	FD_SET(sockets[1], &read_fds);

	struct timeval timeout;
	timeout.tv_sec = 0;
	timeout.tv_usec = 300*1000;

	const int r = select(sockets[1] + 1, &read_fds, 0, 0, &timeout);
	if (r > 0)
	{
	    if (FD_ISSET(sockets[1], &read_fds))
	    {
		unsigned char tmp[sizeof(UartMultidropProtocol::Frame)];
		const int r = read(sockets[1], tmp, sizeof(tmp));
		if (r > 0)
		{
		    UartMultidropProtocol::Frame *frame = (UartMultidropProtocol::Frame *)tmp;
		    size_t remain = frame->size;
		    const unsigned char * data = frame->data;
		    while (remain > 3)
		    {
			UartMultidropProtocol::Frame frame1;
			switch (data[0])
			{
			case 'B': // Button
			    printf ("button: %2X:%d.%d = %d\n", frame->source, data[1], data[2], data[3]);
			    build_led_message(frame1, frame->source, data[1], data[2], 15, data[3] ? 1023 : 0);
			    write(sockets[1], &frame1, frame1.frameSize());
			    break;

			case 'F': // Fader
			    printf ("fader: %2X:%d = %d\n", frame->source, data[1], (data[2]<<8) + data[3]);
			    build_led_message(frame1, frame->source, data[1], 3, 15, (data[2]<<8) + data[3]);
			    write(sockets[1], &frame1, frame1.frameSize());
			    break;
			}
			remain -= 4;
			data += 4;
		    }
		}
	    }
	}
	else if (r == 0)
	{
	    printf ("send observer\n");
	    UartMultidropProtocol::Frame frame;
	    build_observer_message(frame);
	    write(sockets[1], &frame, frame.frameSize());
	}
    }
    return 0;
}

